import socket

import click

from src.config.global_config import compile_ip, compile_host_mame


def net_test(host, port):
    try:
        remote_ip = host
        if compile_host_mame.match(host):
            remote_ip = socket.gethostbyname(host)
        elif compile_ip.match(host):
            remote_ip = socket.gethostbyaddr(host)
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((remote_ip, port))
        click.echo("{}的{}端口连接成功!".format(host, port))
        sock.close()
    except socket.error as e:
        click.echo("{}的{}端口连接失败,原因:{}".format(host, port, e))



import os
import time
from urllib.parse import quote

import click
import git
import yaml

from src.config.global_config import root_config_dir, config_default_file

# 默认配置file
sync_config_default_file = root_config_dir + '/sync_config.yaml'


def details():
    checkSyncConfig()
    y_read_file = open(sync_config_default_file, 'r')
    config = yaml.safe_load(y_read_file)
    if config is None:
        click.echo("暂无同步仓库配置信息!")
        return
    else:
        sc = SyncConfig.to_obj(config[0])
        click.echo("仓库地址:{},用户名:{},密码:{}".format(sc.url, sc.username, sc.password))


def save(url, username, password):
    """
    创建配置  多次创建 会依次覆盖
    :param url:  同步仓库url
    :param username:  同步仓库用户名
    :param password:  同步仓库密码
    :return:
    """
    y_file = open(sync_config_default_file, 'w')
    sc = SyncConfig(url=url, username=username, password=password)
    yaml.safe_dump([sc.__dict__], y_file)
    git.Repo(path=config_default_file).create_remote("origin", url)
    click.echo('\n录入的同步仓库信息:\n地址:{},用户名:{},密码:{}'.format(url, username, password))


def remove():
    """
    删除 配置文件
    :return:
    """
    try:
        checkSyncConfig()
        os.unlink(sync_config_default_file)
        git.Repo(path=config_default_file).delete_remote("origin")
    except OSError:
        click.echo("删除同步仓库配置失败")
    else:
        click.echo("删除同步仓库配置")


def pull():
    checkSyncConfig()
    y_read_file = open(sync_config_default_file, 'r')
    config = yaml.safe_load(y_read_file)
    if config is None:
        click.echo("暂无同步仓库配置信息!")
        return
    git.Repo(path=config_default_file).remote().pull()
    click.echo("sync pull config")


def push(remark):
    checkSyncConfig()
    git_par = git.Repo(path=config_default_file).git
    git_par.add('./*')
    # 当工作区不是干净的 就进行 commit -》 push
    commit_str = '\'{},同步jiuming-tools配置,{}\''.format(time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())),
                                                      remark)
    git_par.commit('-m {}'.format(commit_str))
    git_par.push()
    click.echo("推送配置：{}".format(commit_str))


def checkSyncConfig():
    """
    检查 配置文件是否存在
    :return:
    """
    if not os.path.exists(sync_config_default_file):
        click.echo("同步仓库配置不存在,请创建同步配置")
        return


def clone():
    """
    clone 配置到本地
    :return:
    """
    if os.path.exists(config_default_file):
        click.echo("已存在工具配置，目录:{}".format(config_default_file))
        return
    checkSyncConfig()
    y_read_file = open(sync_config_default_file, 'r')
    config = yaml.safe_load(y_read_file)
    if config is None:
        click.echo("暂无同步仓库配置信息!")
        return
    sc = SyncConfig.to_obj(config[0])
    #    git clone http://username:password@remote 为了避免username和password中包含特殊符号 如@  需要进行转码为url的编码
    url = sc.url
    if sc.url.startswith("http://"):
        url = sc.url.replace("http://", "http://{}:{}@".format(quote(sc.username), quote(sc.password)))
    elif sc.url.startswith("https://"):
        url = sc.url.replace("https://", "https://{}:{}@".format(quote(sc.username), quote(sc.password)))
    git.Repo.clone_from(url=url, to_path=config_default_file)
    click.echo("sync clone config by {}".format(sc.url))
    return


class SyncConfig(object):
    """
    同步配置 对象
    暂时只支持 http/https 协议git 操作
    """
    # 地址
    url: str
    # git用户名
    username: str
    # 密码
    password: str

    def __init__(self, url, username, password):
        self.url = url
        self.username = username
        self.password = password

    @staticmethod
    def to_obj(d: dict):
        """
        将读取的dict 转换为 serverConfig
        :param d:  dict
        :return: ServerConfig
        """
        return SyncConfig(url=d['url'], username=d['username'],
                          password=d['password'])



# -*- coding:utf-8 -*-
from setuptools import setup, find_packages

from src.config import global_config

setup(
    name='jiuming-tools',
    # 版本 如果需要发布更新 需要调整版本号
    version=global_config.version,
    packages=find_packages(),
    include_package_data=True,
    platforms='linux',
    # 需要的依赖
    install_requires=[
        'Click==7.0',
        'psutil==5.6.7',
        'PyYAML==5.2',
        'paramiko==2.7.1',
        'gitpython==3.1.1',
        'bcrypt==3.1.7'
    ],
    # 命令行入口
    entry_points='''
        [console_scripts]
        m=src.ming:cli
    ''',
    author='ming',
    license='MIT',
    url='https://github.com/xujiuming/jiuming-tools',
    description='个人常用功能'
)



import html
import io
import os
import sys
import urllib
from functools import partial
from http import HTTPStatus
from http.server import SimpleHTTPRequestHandler, HTTPServer

import click

from src.utils import iputils


def http_server(d, port, host):
    handler = partial(MingHttpHandler, directory=d)
    httpd = HTTPServer((host, port), handler)
    click.echo(
        'http server start.............监听地址:{}:{} \nhttp://{}:{}'.format(host, port, iputils.get_host_ip(), port))
    httpd.serve_forever(poll_interval=0.1)


class MingHttpHandler(SimpleHTTPRequestHandler):

    def list_directory(self, path):
        """
        重写 显示目录的页面  增加 .  ..目录
        修改title
        """
        try:
            list = os.listdir(path)
        except OSError:
            self.send_error(
                HTTPStatus.NOT_FOUND,
                "No permission to list directory")
            return None
        # 由于os.listdir 不显示 .  .. 这里手工为每个文件夹目录添加
        list.append('.')
        list.append('..')
        list.sort(key=lambda a: a.lower())
        r = []
        try:
            displaypath = urllib.parse.unquote(self.path,
                                               errors='surrogatepass')
        except UnicodeDecodeError:
            displaypath = urllib.parse.unquote(path)
        displaypath = html.escape(displaypath, quote=False)
        enc = sys.getfilesystemencoding()
        title = 'Directory listing for %s' % displaypath
        r.append('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" '
                 '"http://www.w3.org/TR/html4/strict.dtd">')
        r.append('<html>\n<head>')
        r.append('<meta http-equiv="Content-Type" '
                 'content="text/html; charset=%s">' % enc)
        r.append('<title>ming-tools %s</title>\n</head>' % title)
        r.append('<body>\n<h1>%s</h1>' % title)
        r.append('<hr>\n<ul>')
        for name in list:
            fullname = os.path.join(path, name)
            displayname = linkname = name
            # Append / for directories or @ for symbolic links
            if os.path.isdir(fullname):
                displayname = name + "/"
                linkname = name + "/"
            if os.path.islink(fullname):
                displayname = name + "@"
                # Note: a link to a directory displays with @ and links with /
            r.append('<li><a href="%s">%s</a></li>'
                     % (urllib.parse.quote(linkname,
                                           errors='surrogatepass'),
                        html.escape(displayname, quote=False)))
        r.append('</ul>\n<hr>\n</body>\n</html>\n')
        encoded = '\n'.join(r).encode(enc, 'surrogateescape')
        f = io.BytesIO()
        f.write(encoded)
        f.seek(0)
        self.send_response(HTTPStatus.OK)
        self.send_header("Content-type", "text/html; charset=%s" % enc)
        self.send_header("Content-Length", str(len(encoded)))
        self.end_headers()
        return f



# -*- coding:utf-8 -*-

import click

from src.config import global_config, config_manager
from src.config.global_config import compile_ip, compile_host_mame
from src.local import http_server, pc_info, net_manager
from src.script import script_manager
from src.server import server_config


def validate_ip_or_host_name_type(ctx, param, value):
    err_msg = '{}不符合ip/域名格式!请检查后输入'.format(value)
    try:
        if compile_ip.match(value):
            return value
        elif compile_host_mame.match(value):
            return value
        else:
            raise click.BadParameter(err_msg)
    except ValueError:
        raise click.BadParameter(err_msg)


def print_version(ctx, param, value):
    """
    输出工具版本
    :param ctx:   click上下文
    :param param: 参数
    :param value:  值
    :return:
    """
    if not value or ctx.resilient_parsing:
        return

    version_info = """
    作者:ming 
    仅适用linux 其他平台兼容性不做保证
    启用自动补全:
    bash:在.bashrc末尾添加 eval "$(_M_COMPLETE=source m)"
    zsh:在.zshrc末尾添加 eval "$(_M_COMPLETE=source_zsh m)"
    jiuming-tools Version {}""".format(global_config.version)
    click.echo(version_info)
    ctx.exit()


@click.group()
@click.option('--version', '-v', help='工具版本', is_flag=True, callback=print_version, expose_value=False, is_eager=True)
def cli():
    pass


# ---------------------- server tools ----------------------------------------------------------------------------------
@cli.group(help='远程服务器管理')
def server():
    pass


@server.command("list", help='显示所有服务器配置')
def server_list():
    server_config.server_list()


@server.command("add", help='添加服务器配置')
@click.option('--name', '-n', prompt='请输入服务器名称')
@click.option('--host', '-h', prompt='请输入服务器地址', callback=validate_ip_or_host_name_type)
@click.option('--port', '-p', prompt='请输入服务器ssh端口,默认为22', default=22)
@click.option('--username', '-u', prompt='请输入服务器用户名')
@click.option('--password', '-pwd', prompt='请输入密码')
def server_add(name, host, port, username, password):
    server_config.server_add(str(name).strip(), host, port, username, password)


@server.command("remove", help='根据名称删除服务器配置')
@click.option('--name', '-n', type=str, prompt='请输入服务器名称', help='服务器名称')
def server_remove(name):
    server_config.server_remove(name)


@server.command('edit', help='使用vi编辑服务器配置')
def server_edit():
    server_config.server_edit()


@server.command('connect', help='🔗连接服务器')
@click.option('--name', '-n', type=str, prompt='请输入服务器名称', help='服务器名称')
def server_connect(name):
    server_config.server_connect(name)


# ----------------------------------- local tools ----------------------------------------------------------------------

@cli.group(help='本机使用的工具')
def local():
    pass


@local.command('pc', help='电脑配置')
def local_pc_info():
    pc_info.echo_pc_info()


@local.command('http', help='根据指定文件夹开启临时http服务器')
@click.option('--d', '-d', type=click.Path(exists=True), default='.', nargs=1, help='指定静态文件目录,默认为.')
@click.option('--port', '-p', default=20000, type=int, nargs=1, help='指定服务端口,默认为20000')
@click.option('--host', '-h', default='0.0.0.0', callback=validate_ip_or_host_name_type, type=str, nargs=1,
              help='指定服务监听地址,默认为0.0.0.0')
def local_tmp_http(d, port, host):
    http_server.http_server(d, port, host)


@local.command('socket-test', help='测试服务器是否可以打开socket')
@click.option('--host', '-h', type=str, prompt='请输入服务器地址', callback=validate_ip_or_host_name_type, help='服务器地址')
@click.option('--port', '-p', type=int, default=80, help='探测端口号(默认为80)')
def net_test(host, port):
    net_manager.net_test(host, port)


# ----------------------------------- tools config manager  -----------------------------------------------------------

config_remark = """
配置管理 \n
使用私有git仓库作为配置保存\n  
如github 私有仓库等 \n
"""


@cli.group(help=config_remark)
def config():
    pass


@config.command('details', help='查看当前配置仓库配置')
def config_details():
    config_manager.details()


@config.command('save', help='创建当前配置仓库配置')
@click.option('--url', '-url', type=str, prompt='同步仓库url地址')
@click.option('--username', '-u', type=str, prompt='同步仓库用户名')
@click.option('--password', '-p', type=str, prompt='同步仓库密码')
def config_save(url, username, password):
    config_manager.save(url=url, username=username, password=password)


@config.command('remove', help='删除当前配置仓库配置')
def config_remove():
    config_manager.remove()


@config.command('pull', help='同步配置远程到本地')
def config_pull():
    config_manager.pull()


@config.command('push', help='同步配置本地到远程仓库')
@click.option('--remark', '-r', type=str, help='推送备注')
def config_push(remark):
    config_manager.push(remark)


@config.command('clone', help='clone配置到本地')
def config_clone():
    config_manager.clone()


# ----------------------------------- tools config manager  -----------------------------------------------------------
# linux 各种脚本管理   shell 、py 等脚本


@cli.group(help='管理常用linux的脚本')
def script():
    pass


@script.command('remove', help='删除脚本')
@click.option('--name', '-n', prompt='脚本名称')
def script_remove(name):
    script_manager.script_remove(name)


@script.command('create', help='创建脚本')
@click.option('--type', '-t', prompt='脚本类型(执行引擎名称)')
@click.option('--name', '-n', prompt='脚本名称')
@click.option('--remark', '-r', prompt='备注描述')
def script_create(type, name, remark):
    script_manager.script_create(type, name, remark)


@script.command('details', help='查看脚本详情')
@click.option('--name', '-n', prompt='脚本名称')
def script_details(name):
    script_manager.script_details(name)


@script.command('list', help='列出当前脚本')
def script_list():
    script_manager.script_list()


@script.command('exec', help='执行脚本')
@click.option('--name', '-n', prompt='脚本名称')
def script_exec(name):
    script_manager.script_exec(name)


# main 函数
if __name__ == '__main__':
    cli()
